@@ -98,9 +98,13 @@ SIGNATURES = {
         ref=([udev_enumerate_p], udev_enumerate_p),
         unref=([udev_enumerate_p], None),
         add_match_subsystem=([udev_enumerate_p, c_char_p], c_int),
+        add_nomatch_subsystem=([udev_enumerate_p, c_char_p], c_int),
         add_match_property=([udev_enumerate_p, c_char_p, c_char_p], c_int),
+        add_match_sysattr=([udev_enumerate_p, c_char_p, c_char_p], c_int),
+        add_nomatch_sysattr=([udev_enumerate_p, c_char_p, c_char_p], c_int),
         add_match_tag=([udev_enumerate_p, c_char_p], c_int),
         add_match_sysname=([udev_enumerate_p, c_char_p], c_int),
+        add_match_parent=([udev_enumerate_p, udev_device_p], c_int),
         add_match_is_initialized=([udev_enumerate_p], c_int),
         scan_devices=([udev_enumerate_p], c_int),
         get_list_entry=([udev_enumerate_p], udev_list_entry_p)),
@@ -138,7 +142,8 @@ SIGNATURES = {
         get_devlinks_list_entry=([udev_device_p], udev_list_entry_p),
         get_tags_list_entry=([udev_device_p], udev_list_entry_p),
         get_properties_list_entry=([udev_device_p], udev_list_entry_p),
-        get_sysattr_list_entry=([udev_device_p], udev_list_entry_p)),
+        get_sysattr_list_entry=([udev_device_p], udev_list_entry_p),
+        has_tag=([udev_device_p, c_char_p], c_int)),
     # monitoring
     'udev_monitor': dict(
         ref=([udev_monitor_p], udev_monitor_p),
@@ -146,6 +151,7 @@ SIGNATURES = {
         new_from_netlink=([udev_p, c_char_p], udev_monitor_p),
         new_from_socket=([udev_p, c_char_p], udev_monitor_p),
         enable_receiving=([udev_monitor_p], c_int),
+        set_receive_buffer_size=([udev_monitor_p], c_int),
         get_fd=([udev_monitor_p], c_int),
         receive_device=([udev_monitor_p], udev_device_p),
         filter_add_match_subsystem_devtype=(
@@ -154,6 +160,24 @@ SIGNATURES = {
     }
 
 
+ERRNO_EXCEPTIONS = {ENOMEM: MemoryError, EOVERFLOW: OverflowError}
+
+
+def exception_from_errno(errno):
+    """
+    Create an exception from ``errno``.
+
+    ``errno`` is an integral error number.
+
+    Return an exception object appropriate to ``errno``.
+    """
+    exception = ERRNO_EXCEPTIONS.get(errno)
+    if exception is not None:
+        return exception()
+    else:
+        return EnvironmentError(errno, os.strerror(errno))
+
+
 def check_negative_errorcode(result, func, *args):
     """
     Error checker for udev funtions, which return negative error codes.
@@ -169,23 +193,57 @@ def check_negative_errorcode(result, func, *args):
     """
     if result < 0:
         # udev returns the *negative* errno code at this point
-        errorcode = -result
-        exceptions = {ENOMEM: MemoryError, EOVERFLOW: OverflowError}
-        exception = exceptions.get(errorcode)
-        if exception is not None:
-            raise exception()
-        else:
-            raise EnvironmentError(errorcode, os.strerror(errorcode))
+        errno = -result
+        raise exception_from_errno(errno)
     else:
         return result
 
 
+def check_errno(result, func, *args):
+    """
+    Error checker to check the system ``errno`` as returned by
+    :func:`ctypes.get_errno()`.
+
+    If ``result`` is not ``0``, an exception according to this errno is raised.
+    Otherwise nothing happens.
+    """
+    if result != 0:
+        errno = get_errno()
+        if errno != 0:
+            raise exception_from_errno(errno)
+    return result
+
+
+def check_errno_on_null_pointer(result, func, *args):
+    """
+    Error checker to check the system ``errno`` as returned by
+    :func:`ctypes.get_errno()`.
+
+    If ``result`` is a null pointer, an exception according to this errno is
+    raised.  Otherwise nothing happens.
+    """
+    if not result:
+        errno = get_errno()
+        if errno != 0:
+            raise exception_from_errno(errno)
+    return result
+
+
 ERROR_CHECKERS = dict(
+    udev_enumerate_add_match_parent=check_negative_errorcode,
     udev_enumerate_add_match_subsystem=check_negative_errorcode,
+    udev_enumerate_add_nomatch_subsystem=check_negative_errorcode,
     udev_enumerate_add_match_property=check_negative_errorcode,
+    udev_enumerate_add_match_sysattr=check_negative_errorcode,
+    udev_enumerate_add_nomatch_sysattr=check_negative_errorcode,
     udev_enumerate_add_match_tag=check_negative_errorcode,
     udev_enumerate_add_match_sysname=check_negative_errorcode,
     udev_enumerate_add_match_is_initialized=check_negative_errorcode,
+    udev_monitor_set_receive_buffer_size=check_errno,
+    # libudev doc says, enable_receiving returns a negative errno, but tests
+    # show that this is not reliable, so query the real error code
+    udev_monitor_enable_receiving=check_errno,
+    udev_monitor_receive_device=check_errno_on_null_pointer,
     udev_monitor_filter_add_match_subsystem_devtype=check_negative_errorcode,
     udev_monitor_filter_add_match_tag=check_negative_errorcode)
 
@@ -220,7 +278,3 @@ def load_udev_library():
 
 
 libudev = load_udev_library()
-
-
-def get_libudev_errno():
-    return get_errno()
