@@ -119,7 +119,7 @@ class Context(object):
         The run runtime directory path defaulting to ``/run`` as unicode
         string.
 
-        .. udevminversion:: 167
+        .. udevversion:: 167
 
         .. versionadded:: 0.10
         """
@@ -206,7 +206,6 @@ class Enumerator(object):
             raise TypeError('Invalid context object')
         self.context = context
         self._as_parameter_ = libudev.udev_enumerate_new(context)
-        self._parents = []
 
     def __del__(self):
         libudev.udev_enumerate_unref(self)
@@ -221,7 +220,9 @@ class Enumerator(object):
         - The value for the keyword argument ``sys_name`` is forwared to
           :meth:`match_sys_name()`.
         - The value for the keyword argument ``tag`` is forwared to
-          :meth:`match_tag()`
+          :meth:`match_tag()`.
+        - The value for the keyword argument ``parent`` is forwared to
+          :meth:`match_parent()`.
         - All other keyword arguments are forwareded one by one to
           :meth:`match_property()`.  The keyword argument itself is interpreted
           as property name, the value of the keyword argument as the property
@@ -233,6 +234,9 @@ class Enumerator(object):
         Return the instance again.
 
         .. versionadded:: 0.8
+
+        .. versionchanged:: 0.13
+           Added ``parent`` keyword
         """
         subsystem = kwargs.pop('subsystem', None)
         if subsystem is not None:
@@ -243,21 +247,28 @@ class Enumerator(object):
         tag = kwargs.pop('tag', None)
         if tag is not None:
             self.match_tag(tag)
+        parent = kwargs.pop('parent', None)
+        if parent is not None:
+            self.match_parent(parent)
         for property, value in kwargs.iteritems():
             self.match_property(property, value)
         return self
 
-    def match_subsystem(self, subsystem):
+    def match_subsystem(self, subsystem, nomatch=False):
         """
         Include all devices, which are part of the given ``subsystem``.
 
-        ``subsystem`` is either a unicode string or a byte string,
-        containing the name of the subsystem.
+        ``subsystem`` is either a unicode string or a byte string, containing
+        the name of the subsystem.  If ``nomatch`` is ``True`` (default is
+        ``False``), the match is inverted:  A device is only included if it is
+        *not* part of the given ``subsystem``.
 
         Return the instance again.
         """
-        libudev.udev_enumerate_add_match_subsystem(
-            self, ensure_byte_string(subsystem))
+        match = (libudev.udev_enumerate_add_match_subsystem
+                 if not nomatch else
+                 libudev.udev_enumerate_add_nomatch_subsystem)
+        match(self, ensure_byte_string(subsystem))
         return self
 
     def match_sys_name(self, sys_name):
@@ -286,7 +297,7 @@ class Enumerator(object):
         - :func:`bool`
         - A byte string
         - Anything convertable to a unicode string (including a unicode string
-          itself :)
+          itself)
 
         Return the instance again.
         """
@@ -294,6 +305,40 @@ class Enumerator(object):
             self, ensure_byte_string(property), property_value_to_bytes(value))
         return self
 
+    def match_attribute(self, attribute, value, nomatch=False):
+        """
+        Include all devices, whose ``attribute`` has the given ``value``.
+
+        ``attribute`` is either a unicode string or a byte string, containing
+        the name of a sys attribute to match.  ``value`` is an attribute value,
+        being one of the following types:
+
+        - :func:`int`,
+        - :func:`bool`
+        - A byte string
+        - Anything convertable to a unicode string (including a unicode string
+          itself)
+
+        If ``nomatch`` is ``True`` (default is ``False``), the match is
+        inverted:  A device is include if the ``attribute`` does *not* match
+        the given ``value``.
+
+        .. note::
+
+           If ``nomatch`` is ``True``, devices which do not have the given
+           ``attribute`` at all are also included.  In other words, with
+           ``nomatch=True`` the given ``attribute`` is *not* guaranteed to
+           exist on all returned devices.
+
+        Return the instance again.
+        """
+        match = (libudev.udev_enumerate_add_match_sysattr
+                 if not nomatch else
+                 libudev.udev_enumerate_add_nomatch_sysattr)
+        match(self, ensure_byte_string(attribute),
+              property_value_to_bytes(value))
+        return self
+
     def match_tag(self, tag):
         """
         Include all devices, which have the given ``tag`` attached.
@@ -302,7 +347,7 @@ class Enumerator(object):
 
         Return the instance again.
 
-        .. udevminversion:: 154
+        .. udevversion:: 154
 
         .. versionadded:: 0.6
         """
@@ -323,21 +368,28 @@ class Enumerator(object):
 
         .. seealso:: :attr:`Device.is_initialized`
 
-        .. udevminversion:: 165
+        .. udevversion:: 165
 
         .. versionadded:: 0.8
         """
         libudev.udev_enumerate_add_match_is_initialized(self)
         return self
 
-    def match_children(self, device):
+    def match_parent(self, parent):
         """
-        Include all *direct* children of the given ``device``.  A child is a
-        device, whose :attr:`Device.parent` points to ``device``.
+        Include all devices on the subtree of the given ``parent`` device.
+
+        The ``parent`` device itself is also included.
+
+        ``parent`` is a :class:`~pyudev.Device`.
 
         Return the instance again.
+
+        .. udevversion:: 172
+
+        .. versionadded:: 0.13
         """
-        self._parents.append(device)
+        libudev.udev_enumerate_add_match_parent(self, parent)
         return self
 
     def __iter__(self):
@@ -349,7 +401,4 @@ class Enumerator(object):
         libudev.udev_enumerate_scan_devices(self)
         entry = libudev.udev_enumerate_get_list_entry(self)
         for name, _ in udev_list_iterate(entry):
-            device = Device.from_sys_path(self.context, name)
-            if (not self._parents) or any(device.parent == p for p
-                                          in self._parents):
-                yield device
+            yield Device.from_sys_path(self.context, name)
