@@ -29,7 +29,7 @@ from __future__ import (print_function, division, unicode_literals,
                         absolute_import)
 
 import os
-from collections import Mapping
+from collections import Mapping, Container, Iterable
 from datetime import timedelta
 
 from pyudev._libudev import libudev
@@ -37,7 +37,7 @@ from pyudev._util import (ensure_byte_string, ensure_unicode_string,
                           udev_list_iterate, string_to_bool)
 
 
-__all__ = ['Device', 'Attributes',
+__all__ = ['Device', 'Attributes', 'Tags',
            'DeviceNotFoundError', 'DeviceNotFoundAtPathError',
            'DeviceNotFoundByNameError', 'DeviceNotFoundByNumberError',
            'DeviceNotFoundInEnvironmentError']
@@ -317,7 +317,7 @@ class Device(Mapping):
         Raise :exc:`DeviceNotFoundInEnvironmentError`, if no device could be
         created from the environment.
 
-        .. udevminversion:: 152
+        .. udevversion:: 152
 
         .. versionadded:: 0.6
         """
@@ -362,9 +362,15 @@ class Device(Mapping):
 
         Return an iterable yielding a :class:`Device` object for each direct
         child of this device.
+
+        .. udevversion:: 172
+
+        .. versionchanged:: 0.13
+           Requires udev version 172 now.
         """
-        for device in self.context.list_devices().match_children(self):
-            yield device
+        for device in self.context.list_devices().match_parent(self):
+            if device != self:
+                yield device
 
     def find_parent(self, subsystem, device_type=None):
         """
@@ -559,7 +565,7 @@ class Device(Mapping):
 
         .. seealso:: :attr:`time_since_initialized`
 
-        .. udevminversion:: 165
+        .. udevversion:: 165
 
         .. versionadded:: 0.8
         """
@@ -576,7 +582,7 @@ class Device(Mapping):
 
         .. seealso:: :attr:`is_initialized`
 
-        .. udevminversion:: 165
+        .. udevversion:: 165
 
         .. versionadded:: 0.8
         """
@@ -629,15 +635,31 @@ class Device(Mapping):
     @property
     def tags(self):
         """
-        An iterator, which yields all attached tags as unicode strings.
+        A :class:`Tags` object representing the tags attached to this device.
 
-        .. udevminversion:: 154
+        The :class:`Tags` object supports a test for a single tag as well as
+        iteration over all tags:
+
+        >>> 'systemd' in device.tags
+        True
+        >>> list(device.tags)
+        [u'systemd', u'seat']
+
+        Tags are arbitrary classifiers that can be attached to devices by udev
+        scripts and daemons.  For instance, systemd_ uses tags for multi-seat_
+        support.
+
+        .. _systemd: http://freedesktop.org/wiki/Software/systemd
+        .. _multi-seat: http://www.freedesktop.org/wiki/Software/systemd/multiseat
+
+        .. udevversion:: 154
 
         .. versionadded:: 0.6
+
+        .. versionchanged:: 0.13
+           Return a :class:`Tags` object now.
         """
-        tags = libudev.udev_device_get_tags_list_entry(self)
-        for tag, _ in udev_list_iterate(tags):
-            yield ensure_unicode_string(tag)
+        return Tags(self)
 
     def __iter__(self):
         """
@@ -737,6 +759,46 @@ class Device(Mapping):
         raise TypeError('Device not orderable')
 
 
+class Tags(Iterable, Container):
+    """
+    A iterable over :class:`Device` tags.
+
+    Subclasses the ``Container`` and the ``Iterable`` ABC.
+    """
+
+    def __init__(self, device):
+        self.device = device
+
+    if hasattr(libudev, 'udev_device_has_tag'):
+        def _has_tag(self, tag):
+            return bool(libudev.udev_device_has_tag(
+                self.device, ensure_byte_string(tag)))
+    else:
+        def _has_tag(self, tag):
+            return any(t == tag for t in self)
+
+    def __contains__(self, tag):
+        """
+        Check for existence of ``tag``.
+
+        ``tag`` is a tag as unicode string.
+
+        Return ``True``, if ``tag`` is attached to the device, ``False``
+        otherwise.
+        """
+        return self._has_tag(tag)
+
+    def __iter__(self):
+        """
+        Iterate over all tags.
+
+        Yield each tag as unicode string.
+        """
+        tags = libudev.udev_device_get_tags_list_entry(self.device)
+        for tag, _ in udev_list_iterate(tags):
+            yield ensure_unicode_string(tag)
+
+
 def _is_attribute_file(filepath):
     """
     Check, if ``filepath`` points to a valid udev attribute filename.
