@@ -29,12 +29,12 @@
 from __future__ import (print_function, division, unicode_literals,
                         absolute_import)
 
-import os
+import sys
 import select
 from contextlib import closing
 
-from pyudev._libudev import libudev, get_libudev_errno
-from pyudev._util import ensure_byte_string, ensure_unicode_string
+from pyudev._libudev import libudev
+from pyudev._util import ensure_byte_string, ensure_unicode_string, reraise
 
 from pyudev.core import Device
 
@@ -82,6 +82,11 @@ class Monitor(object):
         self._as_parameter_ = monitor_p
         self._socket_path = socket_path
 
+    def _reraise_with_socket_path(self):
+        _, exc_value, traceback = sys.exc_info()
+        exc_value.filename = self._socket_path
+        reraise(exc_value, traceback)
+
     def __del__(self):
         libudev.udev_monitor_unref(self)
 
@@ -189,7 +194,7 @@ class Monitor(object):
 
         This method must be called *before* :meth:`enable_receiving`.
 
-        .. udevminversion:: 154
+        .. udevversion:: 154
 
         .. versionadded:: 0.9
         """
@@ -209,14 +214,43 @@ class Monitor(object):
            need to call it explicitly, if you are iterating over the
            monitor.
         """
-        error = libudev.udev_monitor_enable_receiving(self)
-        if error:
-            errno = get_libudev_errno()
-            raise EnvironmentError(errno, os.strerror(errno),
-                                   self._socket_path)
+        try:
+            libudev.udev_monitor_enable_receiving(self)
+        except EnvironmentError:
+            self._reraise_with_socket_path()
 
     start = enable_receiving
 
+    def set_receive_buffer_size(self, size):
+        """
+        Set the receive buffer ``size``.
+
+        ``size`` is the requested buffer size in bytes, as integer.
+
+        .. note::
+
+           The CAP_NET_ADMIN capability must be contained in the effective
+           capability set of the caller for this method to succeed.  Otherwise
+           :exc:`~exceptions.EnvironmentError` will be raised, with ``errno``
+           set to :data:`~errno.EPERM`.  Unprivileged processes typically lack
+           this capability.  You can check the capabilities of the current
+           process with the python-prctl_ module:
+
+           >>> import prctl
+           >>> prctl.cap_effective.net_admin
+
+        Raise :exc:`~exceptions.EnvironmentError`, if the buffer size could not
+        bet set.
+
+        .. versionadded:: 0.13
+
+        .. _python-prctl: http://packages.python.org/python-prctl
+        """
+        try:
+            libudev.udev_monitor_set_receive_buffer_size(self, size)
+        except EnvironmentError:
+            self._reraise_with_socket_path()
+
     def receive_device(self):
         """
         Receive a single device from the monitor.
@@ -241,13 +275,12 @@ class Monitor(object):
         Raise :exc:`~exceptions.EnvironmentError`, if no device could be
         read.
         """
-        device_p = libudev.udev_monitor_receive_device(self)
+        try:
+            device_p = libudev.udev_monitor_receive_device(self)
+        except EnvironmentError:
+            self._reraise_with_socket_path()
         if not device_p:
-            errno = get_libudev_errno()
-            if errno == 0:
-                raise EnvironmentError('Could not receive device')
-            else:
-                raise EnvironmentError(errno, os.strerror(errno))
+            raise EnvironmentError('Could not receive device')
         action = ensure_unicode_string(
             libudev.udev_device_get_action(device_p))
         return action, Device(self.context, device_p)
