@@ -36,7 +36,7 @@ from pyudev import (Device,
                     DeviceNotFoundByNameError,
                     DeviceNotFoundByNumberError,
                     DeviceNotFoundInEnvironmentError)
-from pyudev.device import Attributes
+from pyudev.device import Attributes, Tags
 
 
 def pytest_generate_tests(metafunc):
@@ -144,12 +144,22 @@ class TestDevice(object):
 
     def test_parent(self, device):
         assert device.parent is None or isinstance(device.parent, Device)
-        if device.parent:
+
+    @pytest.need_udev_version('>= 172')
+    def test_child_of_parent(self, device):
+        if device.parent is None:
+            pytest.skip('Device {0!r} has no parent'.format(device))
+        else:
             assert device in device.parent.children
 
+    @pytest.need_udev_version('>= 172')
     def test_children(self, device):
-        for child in device.children:
-            assert child.parent == device
+        children = list(device.children)
+        if not children:
+            pytest.skip('Device {0!r} has no children'.format(device))
+        else:
+            assert all(c != device for c in children)
+            assert all(c.parent == device for c in children)
 
     def test_find_parent(self, device):
         parent = device.find_parent(device.subsystem)
@@ -157,7 +167,6 @@ class TestDevice(object):
             pytest.xfail('no parent within the same subsystem')
         assert parent.subsystem == device.subsystem
         assert parent in device.traverse()
-        assert device in parent.children
 
     def test_find_parent_no_devtype_mock(self, device):
         get_parent = 'udev_device_get_parent_with_subsystem_devtype'
@@ -192,7 +201,6 @@ class TestDevice(object):
         child = device
         for parent in device.traverse():
             assert parent == child.parent
-            assert child in parent.children
             child = parent
 
     def test_sys_path(self, device, sys_path):
@@ -272,7 +280,7 @@ class TestDevice(object):
         # see TestAttributes for complete attribute tests
         assert isinstance(device.attributes, Attributes)
 
-    def test_attributes_no_leak(self, context):
+    def test_no_leak(self, context):
         """
         Regression test for issue #32, modelled after the script which revealed
         this issue.
@@ -289,26 +297,9 @@ class TestDevice(object):
         # make sure that no memory leaks
         assert not gc.garbage
 
-    @pytest.need_udev_version('>= 154')
-    def test_device_tags(self, device):
-        try:
-            is_input_device = (device.asbool('ID_INPUT_MOUSE') or
-                               device.asbool('ID_INPUT_KEYBOARD'))
-        except KeyError:
-            is_input_device = False
-        if is_input_device:
-            assert 'seat' in device.tags
-        else:
-            pytest.skip('no tags known for device')
-
-    @pytest.need_udev_version('>= 154')
-    def test_device_tags_mock(self, device):
-        tags = [b'spam', b'eggs', b'foo', b'bar']
-        get_tags_list_entry = 'udev_device_get_tags_list_entry'
-        with pytest.patch_libudev_list(tags, get_tags_list_entry):
-            device_tags = list(device.tags)
-            assert device_tags == ['spam', 'eggs', 'foo', 'bar']
-            assert all(pytest.is_unicode_string(t) for t in device_tags)
+    def test_tags(self, device):
+        # see TestTags for complete tag tests
+        assert isinstance(device.tags, Tags)
 
     def test_iteration(self, device, properties):
         device_properties = set(device)
@@ -394,7 +385,7 @@ class TestAttributes(object):
         with pytest.patch_libudev_list(attributes,
                                        get_sysattr_list) as get_sysattr_list:
             attrs = list(device.attributes)
-            attrs == ['spam', 'eggs']
+            assert attrs == ['spam', 'eggs']
             get_sysattr_list.assert_called_with(device)
 
     def test_contains(self, device, attributes):
@@ -432,3 +423,26 @@ class TestAttributes(object):
                     device.attributes.asbool(attribute)
                 message = 'Not a boolean value: {0!r}'
                 assert str(exc_info.value) == message.format(value)
+
+
+class TestTags(object):
+
+    pytestmark = pytest.need_udev_version('>= 154')
+
+    def test_iteration(self, device, tags):
+        assert set(device.tags) == set(tags)
+        assert all(pytest.is_unicode_string(t) for t in device.tags)
+
+    def test_contains(self, device, tags):
+        assert all(t in device.tags for t in tags)
+
+    @pytest.need_udev_version('>= 172')
+    def test_contains_mock(self, device):
+        """
+        Test that ``udev_device_has_tag`` is called if available.
+        """
+        has_tag = 'udev_device_has_tag'
+        with pytest.patch_libudev(has_tag) as has_tag:
+            has_tag.return_value = 1
+            assert 'foo' in device.tags
+            has_tag.assert_called_with(device, b'foo')
